@@ -132,26 +132,18 @@ bytesPerLineTable (const Header &header,
     return maxBytesPerLine;
 }
 
-
-const int&
-sampleCount(const char* base, int xStride, int yStride, int x, int y)
+static int
+roundToNextMultiple(int n, int d)
 {
-    const char* ptr = base + y * yStride + x * xStride;
-    int* intPtr = (int*) ptr;
-
-    return *intPtr;
+    return ((n + d - 1) / d) * d;
 }
 
-int&
-sampleCount(char* base, int xStride, int yStride, int x, int y)
+static int
+roundToPrevMultiple(int n, int d)
 {
-    char* ptr = base + y * yStride + x * xStride;
-    int* intPtr = (int*) ptr;
-    
-    return *intPtr;
+    return (n / d) * d;
 }
 
-
 size_t
 bytesPerDeepLineTable (const Header &header,
                        int minY, int maxY,
@@ -167,18 +159,33 @@ bytesPerDeepLineTable (const Header &header,
          c != channels.end();
          ++c)
     {
-        for (int y = minY; y <= maxY; ++y)
-            if (modp (y, c.channel().ySampling) == 0)
+        const int ySampling = abs(c.channel().ySampling);
+        const int xSampling = abs(c.channel().xSampling);
+        const int pixelSize = pixelTypeSize (c.channel().type);
+
+        // Here we transform from the domain over all pixels into the domain
+        // of actual samples.  We want to sample points in [minY, maxY] where
+        // (y % ySampling) == 0.  However, doing this by rejecting samples
+        // requires O(height*width) modulo computations, which were a
+        // significant bottleneck in the previous implementation of this
+        // function.  For the low, low price of 4 divisions per channel, we
+        // can tighten the y & x ranges to the least and greatest roots of the
+        // sampling function and then stride by the sampling rate.
+        const int sampleMinY = roundToNextMultiple(minY, ySampling);
+        const int sampleMaxY = roundToPrevMultiple(maxY, ySampling);
+        const int sampleMinX = roundToNextMultiple(dataWindow.min.x, xSampling);
+        const int sampleMaxX = roundToPrevMultiple(dataWindow.max.x, xSampling);
+
+        for (int y = sampleMinY; y <= sampleMaxY; y+=ySampling)
+        {
+            int nBytes = 0;
+            for (int x = sampleMinX; x <= sampleMaxX; x += xSampling)
             {
-                int nBytes = 0;
-                for (int x = dataWindow.min.x; x <= dataWindow.max.x; x++)
-                {
-                    if (modp (x, c.channel().xSampling) == 0)
-                        nBytes += pixelTypeSize (c.channel().type) *
-                                  sampleCount(base, xStride, yStride, x, y);
-                }
-                bytesPerLine[y - dataWindow.min.y] += nBytes;
+                nBytes += pixelSize *
+                          sampleCount(base, xStride, yStride, x, y);
             }
+            bytesPerLine[y - dataWindow.min.y] += nBytes;
+        }
     }
 
     size_t maxBytesPerLine = 0;
@@ -1375,6 +1382,20 @@ skipChannel (const char *& readPtr,
 }
 
 
+namespace
+{
+//
+// helper function to realign floats
+// for architectures that require 32-bit alignment for float reading
+//
+
+struct FBytes { uint8_t b[4]; };
+union bytesOrFloat {
+  FBytes b;
+  float f;
+} ;
+}
+
 void
 convertInPlace (char *& writePtr,
                 const char *& readPtr,
@@ -1405,7 +1426,9 @@ convertInPlace (char *& writePtr,
     
         for (size_t j = 0; j < numPixels; ++j)
         {
-            Xdr::write <CharPtrIO> (writePtr, *(const float *) readPtr);
+            union bytesOrFloat tmp;
+            tmp.b = * reinterpret_cast<const FBytes *>( readPtr );
+            Xdr::write <CharPtrIO> (writePtr, tmp.f);
             readPtr += sizeof(float);
         }
         break;
